/*
 * Created on Apr 13, 2005
 *
 */
package com.sunlabs.archsim.fleet.HornAndFunnel;

import com.sunlabs.archsim.base.hierarchy.CompositeEntity;
import com.sunlabs.archsim.fleet.common.FleetGlobal;
import com.sunlabs.archsim.fleet.components.hornAndFunnelFabric.TestInstrGen;
import com.sunlabs.archsim.fleet.components.hornAndFunnelFabric.TestShip;
import com.sunlabs.archsim.gasp.State;

/**
 *
 * Copyright  2005 Sun Microsystems Inc. All rights reserved. Use is subject to license terms.
 * 
 * @author ib27688
 *
 * A test fleet fabric.
 */
public class TestFleet extends CompositeEntity {
	
	public static final String SHIP= "ship";
	public static final String INSTRUCTION_HORN= "instructionHorn";
	public static final String SOURCE_FUNNEL= "sourceFunnel";
	public static final String DESTINATION_HORN= "destinationHorn";
	public static final String TRUNK= "trunk";
	public static final String TRUNK_ACK= "trunkAck";
	public static final String INSTR_GEN= "instrGen";
	public static final String INSTR_IN= "InstructionTrunk";
	public static final String ACK_INSTR_IN= "ackInstructiontrunk";
	
	public static final String INSTR_TOKEN= "InstrToken";
	public static final String ACK_INSTR_TOKEN= "ackInstrToken";
	
	public static final String SRC_DATA= "SrcData";
	public static final String ACK_SRC_DATA= "ackSrcData";

	public static final String DEST_DATA= "DestData";
	public static final String ACK_DEST_DATA= "ackDestData";	
	
	public static final String DELIMITER= "_";
	
	TestInstrGen instrGen;

	// states used for sending instructions to instrHorn, and the ack
	State instr;
	State ackInstr;
	
	// standard fleet pieces
	InstructionHorn instrHorn;
	SourceFunnel sourceFunnel;
	State trunk;
	State trunkAck;
	DestinationHorn destHorn;

	// connect instrHorn and sourceFunnel input
	State[] instrTokenState;
	State[] ackInstrTokenState;
	
	// connect shipOutputs (sources) and source funnel input
	State[] srcDataState;
	State[] ackSrcDataState;
	
	// connect destHorn output and ship inputs (destinations)
	State[] destDataState;
	State[] ackDestDataState;
	
	TestShip[] ship;
	
	/** depth of trees */
	int depth;
	/** number of ships */
	int fanout;
	
	public TestFleet(String name, int d) {
		super(name);
		depth= d;
		fanout= 1<<depth;
		
		//make the main "parts"
		instrGen= new TestInstrGen(INSTR_GEN, fanout, null);
		this.addMember(instrGen);
		
		instrHorn= new InstructionHorn(INSTRUCTION_HORN, depth);
		this.addMember(instrHorn);
		
		sourceFunnel= new SourceFunnel(SOURCE_FUNNEL, depth);
		this.addMember(sourceFunnel);
		
		// in this models the trunk are two states 
		trunk= new State(TRUNK);
		this.addMember(trunk);
		// ack must be initially active
		trunkAck= new State(TRUNK_ACK, this);
		trunkAck.setInitiallyActive(null);
		// this.addMember(trunkAck);
		
		destHorn= new DestinationHorn(DESTINATION_HORN, depth);
		this.addMember(destHorn);
		
		// connect instrGen with instrHorn
		instr= new State(INSTR_IN, this);
		// initially active with value null
		ackInstr= new State(ACK_INSTR_IN, this);
		ackInstr.setInitiallyActive(null);
		
		instr.attach(instrGen.getOut());
		instr.attach(instrHorn.getIn());
		ackInstr.attach(instrGen.getAckOut());
		ackInstr.attach(instrHorn.getAckIn());
		
		//  ackInstr.setActive(null);
		
		
		// connect the sourceFunnel and destHorn
		trunk.attach(sourceFunnel.getOut());
		trunk.attach(destHorn.getIn());
		trunkAck.attach(sourceFunnel.getAckOut());
		trunkAck.attach(destHorn.getAckIn());
		
		// make trunkackActive
		// was made active at creation
		// trunkAck.setActive(null);
		
		// set the log for the trunk
		trunk.setJournal(FleetGlobal.LOG_TRUNK);
		trunkAck.setJournal(FleetGlobal.LOG_TRUNK);
		
		// make the ships
		ship= new TestShip[fanout];
		for (int i=0; i< fanout; i++) {
			ship[i]= new TestShip(SHIP+DELIMITER+i, i);
			this.addMember(ship[i]);
		}
		
		// prepare for connections - get the arrays ready
		instrTokenState= new State[fanout];
		ackInstrTokenState= new State[fanout];
		srcDataState= new State[fanout];
		ackSrcDataState= new State[fanout];
		destDataState= new State[fanout];;
		ackDestDataState= new State[fanout];
		
		
		// connect the instruction horn and source funnel
		for (int i= 0; i< fanout; i++) {

			instrTokenState[i]= new State(INSTR_TOKEN
					+DELIMITER+i, this);
			// ack state is initlally active
			ackInstrTokenState[i]= new State(ACK_INSTR_TOKEN
					+DELIMITER+i, this);
			ackInstrTokenState[i].setInitiallyActive(null);
			
			instrTokenState[i].attach(instrHorn.getOut(i));
			instrTokenState[i].attach(sourceFunnel.getIn(i));

			ackInstrTokenState[i].attach(instrHorn.getAckOut(i));
			ackInstrTokenState[i].attach(sourceFunnel.getAckIn(i));
			
			// make the ack state active
			// was made active at creation
			// ackInstrTokenState[i].setActive(null);
		}
		
		// connect sources (ship outputs) and sourceFunnel
		for (int i=0; i< fanout; i++) {
			srcDataState[i]= new State(SRC_DATA+DELIMITER+i, this);
			// ack state is initially active
			ackSrcDataState[i]= new State(ACK_SRC_DATA
					+DELIMITER+i, this);
			ackSrcDataState[i].setInitiallyActive(null);
			
			srcDataState[i].attach(ship[i].getOut());
			srcDataState[i].attach(sourceFunnel.getSrcIn(i));
			
			ackSrcDataState[i].attach(ship[i].getAckOut());
			ackSrcDataState[i].attach(sourceFunnel.getAckSrcIn(i));
			
			// make the ack state active
			// was made active at creation
			// ackSrcDataState[i].setActive(null);
		}
		
		// connect destHorn and destinations (ship inputs)
		for (int i=0; i< fanout; i++) {
			destDataState[i]= new State(DEST_DATA
					+DELIMITER+i, this);
			ackDestDataState[i]= new State(ACK_DEST_DATA
					+DELIMITER+i, this);
			ackDestDataState[i].setInitiallyActive(null);
			
			destDataState[i].attach(destHorn.getOut(i));
			destDataState[i].attach(ship[i].getIn());
			
			ackDestDataState[i].attach(destHorn.getAckOut(i));
			ackDestDataState[i].attach(ship[i].getAckIn());
			
			// make the ack state active
			// was made active at creation
			//ackDestDataState[i].setActive(null);
		}
		
	} // TestFleet

}
